import React, { useState } from 'react'
import PropTypes from 'prop-types'
import Lightbox from 'react-image-lightbox'
import 'react-image-lightbox/style.css'
import classNames from 'classnames'

import Cards, { Card } from './Cards'

const massageQuery = (query) =>
  query.edges.map((edge) => {
    const node = edge.node

    return {
      description: node.description,
      full: node.image.full,
      thumb: node.image.thumb,
      title: node.title,
    }
  })

const ImageModal = ({
  images,
  options,
  close,
  setCurrentImageIndex,
  currentImageIndex,
}) => {
  const lightboxOptions = {
    enableZoom: false,
    discourageDownloads: false,
    ...options,
  }

  const nextImageIndex = () => (currentImageIndex + 1) % images.length
  const previousImageIndex = () =>
    (currentImageIndex + images.length - 1) % images.length
  const imageSource = (index) => images[index].full.fluid.src
  const thumbnailSource = (index) => images[index].thumb.fluid.src
  const imageCaption = (index) => images[index].description
  const imageTitle = (index) => images[index].title

  return (
    <Lightbox
      mainSrc={imageSource(currentImageIndex)}
      nextSrc={imageSource(nextImageIndex())}
      prevSrc={imageSource(previousImageIndex())}
      mainSrcThumbnail={thumbnailSource(currentImageIndex)}
      nextSrcThumbnail={thumbnailSource(nextImageIndex())}
      prevSrcThumbnail={thumbnailSource(previousImageIndex())}
      onCloseRequest={() => close()}
      onMoveNextRequest={() => setCurrentImageIndex(nextImageIndex())}
      onMovePrevRequest={() => setCurrentImageIndex(previousImageIndex())}
      enableZoom={lightboxOptions.enableZoom}
      discourageDownloads={lightboxOptions.discourageDownloads}
      imageTitle={imageTitle(currentImageIndex)}
      imageCaption={imageCaption(currentImageIndex)}
    />
  )
}

const ImageGallery = ({ query, captions = false, className, options = {} }) => {
  const [modalOpen, setModalOpen] = useState(false)
  const [currentImageIndex, setCurrentImageIndex] = useState(0)

  const images = massageQuery(query)
  const galleryClassName = classNames('heimr-gallery', className)

  const openModal = (index) => {
    setCurrentImageIndex(index)
    setModalOpen(true)
  }

  return (
    <>
      <Cards className={galleryClassName}>
        {images.map((image, index) => {
          const cardImage = {
            alt: image.title,
            fluid: image.thumb.fluid,
            title: image.title,
            className: 'heimr-gallery__thumbnail',
          }

          const contentClassName = classNames('heimr-gallery__item', {
            'heimr-gallery__item--portrait': image.thumb.fluid.aspectRatio < 1,
            'heimr-gallery__item--lanscape': image.thumb.fluid.aspectRatio >= 1,
          })

          return (
            <Card
              key={index}
              image={cardImage}
              onClick={() => openModal(index)}
              className={contentClassName}
            >
              {image.title && captions && (
                <p className='heimr-gallery__caption'>{image.title}</p>
              )}
            </Card>
          )
        })}
      </Cards>
      {modalOpen && (
        <ImageModal
          images={images}
          options={{ ...options }}
          close={() => setModalOpen(false)}
          setCurrentImageIndex={setCurrentImageIndex}
          currentImageIndex={currentImageIndex}
        />
      )}
    </>
  )
}

const imagePropTypes = PropTypes.shape({
  fluid: PropTypes.shape({
    aspectRation: PropTypes.number,
    base64: PropTypes.string,
    sizes: PropTypes.string,
    src: PropTypes.string,
    srcSet: PropTypes.string,
    srcSetWebp: PropTypes.string,
    srcWebp: PropTypes.string,
  }),
})

ImageGallery.propTypes = {
  query: PropTypes.shape({
    edges: PropTypes.arrayOf(
      PropTypes.shape({
        node: PropTypes.shape({
          description: PropTypes.string,
          full: imagePropTypes,
          thumb: imagePropTypes,
          title: PropTypes.string,
        }),
      })
    ),
  }).isRequired,
  captions: PropTypes.bool,
}

export default ImageGallery
